/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.model.BaseEntity;
import ru.ulstu.extractor.model.TimeSeries;
import ru.ulstu.extractor.model.TimeSeriesValue;
import ru.ulstu.extractor.repository.TimeSeriesRepository;
import ru.ulstu.extractor.repository.TimeSeriesValueRepository;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class TimeSeriesService {
    private final static Logger LOG = LoggerFactory.getLogger(TimeSeriesService.class);
    private final TimeSeriesRepository timeSeriesRepository;
    private final TimeSeriesValueRepository timeSeriesValueRepository;


    public TimeSeriesService(TimeSeriesRepository timeSeriesRepository, TimeSeriesValueRepository timeSeriesValueRepository) {
        this.timeSeriesRepository = timeSeriesRepository;
        this.timeSeriesValueRepository = timeSeriesValueRepository;
    }

    public TimeSeries save(TimeSeries timeSeries, List<TimeSeriesValue> timeSeriesValues) {
        LOG.debug("Start save {} time series with {} time series values ", timeSeries.getName(), timeSeriesValues.size());
        List<Integer> timeSeriesValuesToRemoveIds = timeSeries.getValues().stream().map(BaseEntity::getId).collect(Collectors.toList());
        timeSeries.setValues(timeSeriesValues);
        LOG.debug("Save time series {} ", timeSeries.getName());
        timeSeries = timeSeriesRepository.save(timeSeries);
        LOG.debug("Clear {} ime series values ", timeSeriesValuesToRemoveIds.size());
        return timeSeries;
    }

    public TimeSeries findOrCreate(String timeSeriesName) {
        Optional<TimeSeries> maybeTimeSeries = timeSeriesRepository.findByName(timeSeriesName);
        if (maybeTimeSeries.isPresent()) {
            LOG.debug("TimeSeries {} exists.", maybeTimeSeries.get().getName());
            return maybeTimeSeries.get();
        }
        return timeSeriesRepository.save(new TimeSeries(timeSeriesName));
    }

    public List<TimeSeriesValue> save(List<TimeSeriesValue> timeSeriesValues) {
        return timeSeriesValues.stream()
                .map(timeSeriesValue -> {
                    timeSeriesValue.setValue((timeSeriesValue.getValue()));
                    timeSeriesValue.setDate((timeSeriesValue.getDate()));
                    return timeSeriesValueRepository.save(timeSeriesValue);
                }).collect(Collectors.toList());
    }

    public void addTimeSeriesValue(String timeSeriesName, Date date, Integer value) {
        LOG.debug("Start add time series values to {} time series values ", timeSeriesName);
        TimeSeries timeSeries = findOrCreate(timeSeriesName);
        timeSeriesValueRepository.save(new TimeSeriesValue(timeSeries, date, value));
    }
//
//    public TimeSeriesValue findByTimeSeriesAndName(TimeSeries timeSeries, String name) {
//        return timeSeriesValueRepository.findByTimeSeriesAndName(timeSeries, name);
//    }

    public List<TimeSeriesValue> findAll() {
        return timeSeriesValueRepository.findAll();
    }
}
